### Note ###
# This file loads the necessary packages and defines custom functions that all 
# other R scripts in this study use. I recommend simply sourcing this file prior
# to running any lines from other scripts. You may need to adjust the max number
# of DLLs that R can handle; see 'set max DLLs' section below

####################### load or install&load packages ##########################
#specify the packages of interest
packages=c("plyr","ggplot2","boot","dplyr","spatialEco","fANCOVA","geomorph",
           "sp","lattice","abind","phytools","viridis","leaflet","dendextend",
           "geiger","splitstackshape","Hmisc","emmeans","xlsx","cluster",
           'MCMCglmm',"nFactors","sensiPhy","spatstat","MuMIn","gghighlight",
           "ggmcmc","caper","ptinpoly","glmulti","caret","mda","lsmeans",
           "sjstats","ggridges","forcats","Momocs","mcmcse","tidybayes","class",
           "reshape2","devtools","rgbif","mapr","spocc","convevol"
           #,"h2o","Cubist" #optional packages
)
#use this function to check if each package is on the local machine
#if a package is installed, it will be loaded
#if any are not, the missing package(s) will be installed and loaded
package.check<-lapply(packages,FUN=function(x){
  if(!require(x,character.only=TRUE)){
    install.packages(x,dependencies=TRUE)
    library(x,character.only=TRUE)
  }
})
#verify they are loaded
search()

#unmask PCA() from Momocs
PCA<-Momocs::PCA

## Compute the mode of a set of data
modeStat<-function(x){
  uniquex<-unique(x)
  uniquex[which.max(tabulate(match(x,uniquex)))]
}

#compute standard error of the mean
stdError <- function(x) sd(x)/sqrt(length(x))

################################ set max DLLs ##################################
## because of the number of packages that are loaded, it may be necessary for 
## you to adjust the maximum number of DLLs that R can handle. please note that
## these settings are machine-specific.

## the following code will do this for computers that use Windows:
#user_renviron=path.expand(file.path("~",".Renviron"))
#if(!file.exists(user_renviron)) # check to see if the file already exists
#  file.create(user_renviron)
#file.edit(user_renviron) # open with another text editor if this fails

#then add the following line (without the '#'), save, and restart R:
#R_MAX_NUM_DLLS=500


################################ session info ##################################
# for reference, here are the details of all package versions used...etc
# acquired using the sessionInfo() function

# > sessionInfo()
# R version 3.5.1 (2018-07-02)
# Platform: x86_64-w64-mingw32/x64 (64-bit)
# Running under: Windows >= 8 x64 (build 9200)
# 
# Matrix products: default
# 
# locale:
# [1] LC_COLLATE=English_United States.1252  LC_CTYPE=English_United States.1252   
# [3] LC_MONETARY=English_United States.1252 LC_NUMERIC=C                          
# [5] LC_TIME=English_United States.1252    
# 
# attached base packages:
# [1] stats     graphics  grDevices utils     datasets  methods   base     
# 
# other attached packages:
#  [1] convevol_1.3          spocc_0.9.0           mapr_0.4.0           
#  [4] rgbif_1.1.0           devtools_1.13.6       reshape2_1.4.3       
#  [7] tidybayes_1.0.1       mcmcse_1.3-2          Momocs_1.2.9.4       
# [10] forcats_0.3.0         ggridges_0.5.1        sjstats_0.17.1       
# [13] lsmeans_2.27-62       mda_0.4-10            class_7.3-14         
# [16] caret_6.0-80          glmulti_1.0.7         rJava_0.9-10         
# [19] ptinpoly_2.4          misc3d_0.8-4          caper_1.0.1          
# [22] mvtnorm_1.0-8         ggmcmc_1.1            tidyr_0.8.1          
# [25] gghighlight_0.1.0     MuMIn_1.42.1          spatstat_1.56-1      
# [28] rpart_4.1-13          nlme_3.1-137          spatstat.data_1.4-0  
# [31] sensiPhy_0.8.3        phylolm_2.6           nFactors_2.3.3       
# [34] psych_1.8.4           MASS_7.3-50           MCMCglmm_2.26        
# [37] coda_0.19-2           Matrix_1.2-14         cluster_2.0.7-1      
# [40] xlsx_0.6.1            emmeans_1.2.4         Hmisc_4.1-1          
# [43] Formula_1.2-3         survival_2.42-3       splitstackshape_1.4.6
# [46] geiger_2.0.6          dendextend_1.8.1      leaflet_2.0.2        
# [49] viridis_0.5.1         viridisLite_0.3.0     phytools_0.6-61      
# [52] maps_3.3.0            abind_1.4-5           lattice_0.20-35      
# [55] sp_1.3-1              geomorph_3.0.7        ape_5.2              
# [58] rgl_0.99.16           fANCOVA_0.5-1         spatialEco_1.1-0     
# [61] dplyr_0.7.6           boot_1.3-20           ggplot2_3.0.0.9000   
# [64] plyr_1.8.4           
# 
# loaded via a namespace (and not attached):
#   [1] estimability_1.3          prabclus_2.2-6            GGally_1.4.0             
#   [4] ModelMetrics_1.2.0        acepack_1.4.1             clusterGeneration_1.3.4  
#   [7] knitr_1.20                multcomp_1.4-8            data.table_1.12.0        
#  [10] generics_0.0.2            TH.data_1.0-9             combinat_0.0-8           
#  [13] future_1.9.0              xml2_1.2.0                webshot_0.5.1            
#  [16] lubridate_1.7.4           httpuv_1.4.5              assertthat_0.2.0         
#  [19] oai_0.2.2                 RRPP_0.3.0                gower_0.1.2              
#  [22] hms_0.4.2                 bayesplot_1.6.0           promises_1.0.1           
#  [25] DEoptimR_1.0-8            igraph_1.2.2              DBI_1.0.0                
#  [28] htmlwidgets_1.3           reshape_0.8.7             tensorA_0.36.1           
#  [31] stringdist_0.9.5.1        ddalpha_1.3.4             stats4_3.5.1             
#  [34] purrr_0.2.5               crosstalk_1.0.0           subplex_1.5-4            
#  [37] backports_1.1.2           trimcluster_0.1-2.1       deldir_0.1-15            
#  [40] pwr_1.2-2                 sjlabelled_1.0.14         withr_2.1.2              
#  [43] sfsmisc_1.1-2             robustbase_0.93-3         checkmate_1.8.5          
#  [46] mclust_5.4.1              goftest_1.1-1             mnormt_1.5-5             
#  [49] dotCall64_1.0-0           lazyeval_0.2.1            crayon_1.3.4             
#  [52] ellipse_0.4.1             recipes_0.1.3             pkgconfig_2.0.2          
#  [55] units_0.6-1               nnet_7.3-12               bindr_0.1.1              
#  [58] rlang_0.2.2               globals_0.12.4            diptest_0.75-7           
#  [61] pls_2.7-0                 miniUI_0.1.1.1            LaplacesDemon_16.1.1     
#  [64] sandwich_2.5-0            modelr_0.1.2              polyclip_1.9-1           
#  [67] rbison_0.7.0              phangorn_2.4.0            zoo_1.8-4                
#  [70] base64enc_0.1-3           whisker_0.3-2             spam_2.2-0               
#  [73] classInt_0.2-3            DRR_0.0.3                 stringr_1.3.1            
#  [76] maptools_0.9-4            manipulateWidget_0.10.0   coin_1.2-2               
#  [79] jpeg_0.1-8                scales_1.0.0              memoise_1.1.0            
#  [82] magrittr_1.5              compiler_3.5.1            RColorBrewer_1.1-2       
#  [85] plotrix_3.7-4             dimRed_0.1.0              lme4_1.1-18-1            
#  [88] snakecase_0.9.2           listenv_0.7.0             TMB_1.7.14               
#  [91] htmlTable_1.12            magic_1.5-9               mgcv_1.8-24              
#  [94] tidyselect_0.2.5          stringi_1.2.4             yaml_2.2.0               
#  [97] svUnit_0.7-12             latticeExtra_0.6-28       rebird_1.0.0             
# [100] grid_3.5.1                fastmatch_1.1-0           tools_3.5.1              
# [103] animation_2.5             future.apply_1.0.1        parallel_3.5.1           
# [106] rstudioapi_0.8            foreach_1.4.4             foreign_0.8-70           
# [109] gridExtra_2.3             cubature_1.4              prodlim_2018.04.18       
# [112] scatterplot3d_0.3-41      rgeos_0.3-28              digest_0.6.18            
# [115] shiny_1.1.0               lava_1.6.3                quadprog_1.5-5           
# [118] fpc_2.1-11.1              bindrcpp_0.2.2            Rcpp_0.12.19             
# [121] broom_0.5.1               later_0.7.5               httr_1.3.1               
# [124] sf_0.6-3                  rvertnet_0.7.0            ggstance_0.3.1           
# [127] kernlab_0.9-27            colorspace_1.3-2          tensor_1.5               
# [130] CVST_0.2-2                fields_9.6                splines_3.5.1            
# [133] RcppRoll_0.3.0            expm_0.999-3              spatstat.utils_1.10-0    
# [136] xlsxjars_0.6.1            mapproj_1.2.6             flexmix_2.3-14           
# [139] spData_0.2.9.4            xtable_1.8-3              jsonlite_1.5             
# [142] nloptr_1.2.1              geometry_0.3-6            corpcor_1.6.9            
# [145] timeDate_3043.102         modeltools_0.2-22         ipred_0.9-7              
# [148] R6_2.3.0                  prediction_0.3.6          pillar_1.3.0             
# [151] htmltools_0.3.6           mime_0.6                  glue_1.3.0               
# [154] minqa_1.2.4               deSolve_1.21              codetools_0.2-15         
# [157] rworldmap_1.3-6           tibble_1.4.2              arrayhelpers_1.0-20160527
# [160] numDeriv_2016.8-1         geoaxe_0.1.0              ridigbio_0.3.5           
# [163] glmmTMB_0.2.2.0           munsell_0.5.0             e1071_1.7-0              
# [166] iterators_1.0.10          sjmisc_2.7.5              haven_1.1.2              
# [169] gtable_0.2.0  