#read in each tree
#codes:
#lab = labrids
#gob = gobies
#dam = damselfishes
#ang = angelfishes
#sfp = surfperches
labTree<-read.nexus("./data/Labridae_BaligaLaw_MCC.tre")
gobTree<-read.nexus("./data/Gobiidae_MCC.tre")
damTree<-read.nexus("./data/Pomacentridae_CowmanBellwood.tre")
angTree<-read.nexus("./data/Pomacanthidae_MCC.tre")
sfpTree<-read.nexus("./data/Embiotocidae_Longo_renamed.tre")


####Import TPS files (.TPS & .NTS)####
#read in tps file with landmarks & semilandmarks
labData<-readland.tps("./data/Labridae.TPS",specID="imageID")
gobData<-readland.tps("./data/Gobiidae.tps",specID="imageID")
damData<-readland.tps("./data/Pomacentridae.TPS",specID="imageID")
angData<-readland.tps("./data/Pomacanthidae.tps",specID="imageID")
sfpData<-readland.tps("./data/Embiotocidae.TPS",specID="imageID")

#read in sliders file; you may need to remove the first two lines using notepad
sliders<-as.matrix(read.table("./data/Sliders_forR.NTS"))

#procrustes superimposition
labSuper <- gpagen(labData,curves=sliders)
labShape <- labSuper$coords
labSize <- labSuper$Csize
gobSuper <- gpagen(gobData,curves=sliders)
gobShape <- gobSuper$coords
gobSize <- gobSuper$Csize
damSuper <- gpagen(damData,curves=sliders)
damShape <- damSuper$coords
damSize <- damSuper$Csize
angSuper <- gpagen(angData,curves=sliders)
angShape <- angSuper$coords
angSize <- angSuper$Csize
sfpSuper <- gpagen(sfpData,curves=sliders)
sfpShape <- sfpSuper$coords
sfpSize <- sfpSuper$Csize

#Outlier detection
labOutlier <- plotOutliers(labShape)
gobOutlier <- plotOutliers(gobShape) 
damOutlier <- plotOutliers(damShape) 
angOutlier <- plotOutliers(angShape) 
sfpOutlier <- plotOutliers(sfpShape) 

####two.d.arrays of data from all individuals####
lab.two.d.inds<-two.d.array(labShape)
gob.two.d.inds<-two.d.array(gobShape)
dam.two.d.inds<-two.d.array(damShape)
ang.two.d.inds<-two.d.array(angShape)
sfp.two.d.inds<-two.d.array(sfpShape)
p<-dim(labShape)[1] # the number of landmarks
k<-dim(labShape)[2] # the dimensions of the coordinates

####read in group IDs####
source('./data/labGroup.R')
source('./data/gobGroup.R')
source('./data/damGroup.R')
source('./data/angGroup.R')
source('./data/sfpGroup.R')

#species means
labMeans<-rowsum(lab.two.d.inds,labGroup)/as.vector(table(labGroup))
gobMeans<-rowsum(gob.two.d.inds,gobGroup)/as.vector(table(gobGroup))
damMeans<-rowsum(dam.two.d.inds,damGroup)/as.vector(table(damGroup))
angMeans<-rowsum(ang.two.d.inds,angGroup)/as.vector(table(angGroup))
sfpMeans<-rowsum(sfp.two.d.inds,sfpGroup)/as.vector(table(sfpGroup))

#Convert matrix (2D) -> array (3D)
lab.three.d<-arrayspecs(labMeans,p,k)
gob.three.d<-arrayspecs(gobMeans,p,k)
dam.three.d<-arrayspecs(damMeans,p,k)
ang.three.d<-arrayspecs(angMeans,p,k)
sfp.three.d<-arrayspecs(sfpMeans,p,k)

#Convert array (3D) -> matrix (2D)
lab.two.d<-two.d.array(lab.three.d)
gob.two.d<-two.d.array(gob.three.d)
dam.two.d<-two.d.array(dam.three.d)
ang.two.d<-two.d.array(ang.three.d)
sfp.two.d<-two.d.array(sfp.three.d)

######NONphyloPCA######
labGeoPCA<-plotTangentSpace(lab.three.d,verbose=FALSE,label=rownames(lab.two.d))
gobGeoPCA<-plotTangentSpace(gob.three.d,verbose=FALSE,label=rownames(gob.two.d))
damGeoPCA<-plotTangentSpace(dam.three.d,verbose=FALSE,label=rownames(dam.two.d))
angGeoPCA<-plotTangentSpace(ang.three.d,verbose=FALSE,label=rownames(ang.two.d))
sfpGeoPCA<-plotTangentSpace(sfp.three.d,verbose=FALSE,label=rownames(sfp.two.d))

#for fun, all families:
all.two.d<-rbind(lab.two.d,gob.two.d,dam.two.d,ang.two.d,sfp.two.d)
all.three.d<-arrayspecs(all.two.d,p,k)
allFamilyFac<-c(rep("Labridae",dim(lab.two.d)[1]),
                rep("Gobiidae",dim(gob.two.d)[1]),
                rep("Pomacentridae",dim(dam.two.d)[1]),
                rep("Pomacanthidae",dim(ang.two.d)[1]),
                rep("Embiotocidae",dim(sfp.two.d)[1]))
all.super<-gpagen(all.three.d,curves=sliders)
all.shape.3D<-all.super$coords
all.shape.2D<-two.d.array(all.shape.3D)
allPCA_geomorph<-plotTangentSpace(all.shape.3D,label=rownames(all.two.d))
allPCA_MOMOCS<-Momocs::PCA(all.shape.2D)
plot(allPCA_MOMOCS$x,pch=19,asp=1,col=c(rep('black',dim(lab.two.d)[1]),
                                        rep('blue',dim(gob.two.d)[1]),
                                        rep('green',dim(dam.two.d)[1]),
                                        rep('yellow',dim(ang.two.d)[1]),
                                        rep('orange',dim(sfp.two.d)[1])))
allPCA_MOMOCS.varExpl<-rbind(
  SD = sqrt(allPCA_MOMOCS$eig),
  Proportion = allPCA_MOMOCS$eig/sum(allPCA_MOMOCS$eig),
  Cumulative = cumsum(allPCA_MOMOCS$eig)/sum(allPCA_MOMOCS$eig))
plot(1:ncol(allPCA_MOMOCS$x),
     allPCA_MOMOCS.varExpl[2,],pch=19);abline(h=mean(allPCA_MOMOCS.varExpl[2,]))
plot(1:ncol(allPCA_MOMOCS.varExpl),allPCA_MOMOCS.varExpl[3,],
     pch=19,ylim=c(0,1));abline(h=0.95)

allPCA_gmrph.varExpl<-rbind(
  SD = allPCA_geomorph$sdev,
  Proportion=((allPCA_geomorph$sdev)^2)/sum(((allPCA_geomorph$sdev)^2)),
  Cumulative=cumsum(((allPCA_geomorph$sdev)^2)/sum(((allPCA_geomorph$sdev)^2))))
plot(1:dim(allPCA_gmrph.varExpl)[2],
     allPCA_gmrph.varExpl[2,],pch=19);abline(h=mean(allPCA_gmrph.varExpl[2,]))
plot(1:ncol(allPCA_gmrph.varExpl),allPCA_gmrph.varExpl[3,],
     pch=19,ylim=c(0,1));abline(h=0.95)

######five fam tree import#####
#to re-create the tree grafting and pruning process, simply source:
source("./tree_prep/tree_grafting.R")
#alternatively, read the grafted trees themselves:
#fiveFamTree<-read.nexus("./tree_prep/fiveFamTree.tre")        #for morphometics
#fiveFamTreeALL<-read.tree("./tree_prep/fiveFamTreeALL.tre") #for diet simmaping

#measure signal in data
fiveFam_K<-physignal(all.shape.3D,fiveFamTree);fiveFam_K
phylosig(fiveFamTree,allPCA_geomorph$pc.scores,method='lambda',test=TRUE)
fiveFam_lambda<-phylosig(fiveFamTree,allPCA_geomorph$pc.scores[,1:30],
                         method='lambda',test=TRUE)

######five fam pPCA#####
fiveFam_pPCA<-phyl.pca(fiveFamTree,allPCA_geomorph$pc.scores[,1:30],
                       method="lambda",opt="fixed",
                       lambda=fiveFam_lambda$lambda)


#plotting
cols.fam<-colorFactor(c(#"#343433", #dark grey
  "#F4E539", #yellow
  #"#F9A31A", #orange
  "#0BB6EA", #light blue
  "#069F77", #green
  #"#F4E539", #yellow
  #"#989897", #medium grey
  #"#0077B7", #dark blue
  "#F16622", #red-orange
  "#E07CAA"  #pink
),
domain=allFamilyFac,ordered=TRUE)

famptls<-setNames(c(#"#343433",
  "#F4E539",
  #"#F9A31A",
  "#0BB6EA",
  "#069F77",
  #"#989897",
  #"#0077B7",
  "#F16622",
  "#E07CAA"),
  c("Labridae","Gobiidae","Pomacentridae","Pomacanthidae",
    "Embiotocidae"))
famtip.cols<-cols.fam(allFamilyFac)
names(famtip.cols)<-rownames(all.two.d)
fam.pmscols<-c(famtip.cols[fiveFamTree$tip.label],rep("black",fiveFamTree$Nnode))
names(fam.pmscols)<-1:(length(fiveFamTree$tip)+fiveFamTree$Nnode)

phylomorphospace(fiveFamTree,fiveFam_pPCA$S[,1:2],label="horizontal",
                 node.size=c(0,1),xlim=c(-0.2,0.2),ylim=c(-0.2,0.2))
phylomorphospace(fiveFamTree,fiveFam_pPCA$S[,1:2],label="none",
                 node.size=c(0,2),control=list(col.node=fam.pmscols)
                 ,xlim=c(-0.2,0.2),ylim=c(-0.2,0.2))

phylomorphospace(fiveFamTree,allPCA_geomorph$pc.scores[,1:2],label="none",
                 node.size=c(0,2),control=list(col.node=fam.pmscols)
                 ,xlim=c(-0.2,0.2),ylim=c(-0.2,0.2))


fiveFam_pPCA.varExpl<-rbind(
  SD = sqrt(rowSums(fiveFam_pPCA$Eval)),
  Proportion = rowSums(fiveFam_pPCA$Eval)/sum(rowSums(fiveFam_pPCA$Eval)),
  Cumulative = cumsum(rowSums(fiveFam_pPCA$Eval))/sum(rowSums(fiveFam_pPCA$Eval)))
plot(1:ncol(fiveFam_pPCA$S),
     fiveFam_pPCA.varExpl[2,],pch=19);abline(h=mean(fiveFam_pPCA.varExpl[2,]))
plot(1:ncol(fiveFam_pPCA.varExpl),fiveFam_pPCA.varExpl[3,],
     pch=19,ylim=c(0,1));abline(h=0.95)

#number of PCs to keep
library(nFactors)
fiveFam_pPCA$S->mydata
ev<-eigen(cov(mydata)) # get eigenvalues
ev$values->a
ap<-nFactors::parallel(subject=nrow(mydata),var=ncol(mydata),
                       sd=diag(a,ncol(mydata)))
nS<-nFactors::nScree(x=ev$values,aparallel=ap$eigen$qevpea)
plotnScree(nS) #retain 3PCs for further analyses
