##### import relevant trees#####
# first import Betancour et al's phylogeny
betanTree<-read.tree("./tree_prep/12862_2017_958_MOESM2_ESM.tre")
#write.table(betanTree$tip.label,"betanTree.txt")
plot(ladderize(betanTree),cex=0.5)
nodelabels(frame="none",cex=0.5)

# read in list of taxa to drop from Betancour tree
source("./tree_prep/taxa-to-drop.R")
backbone<-drop.tip(betanTree,taxa_to_drop)
#writeNexus(ladderize(backbone),"backbone.tre")
plot(ladderize(backbone));nodelabels(frame='none')
axisPhylo(side=1,root.time=NULL,backward=TRUE)
ladderize(backbone)->backbone

#read in family-specific trees (if not already done)
labTree<-read.nexus("./data/Labridae_BaligaLaw_MCC.tre")
gobTree<-read.nexus("./data/Gobiidae_MCC.tre")
damTree<-read.nexus("./data/Pomacentridae_CowmanBellwood.tre")
angTree<-read.nexus("./data/Pomacanthidae_MCC.tre")
sfpTree<-read.nexus("./data/Embiotocidae_Longo_renamed.tre")

#need to remove the outgroup from labTree because it has some tips in common
#with the other families
source("./tree_prep/labrids_taxa-to-drop.R") #labsOutgroup
labTree<-drop.tip(labTree,labsOutgroup)


###### tree grafting ######
# get some basic info
max(branching.times(backbone)) #121.6
max(branching.times(labTree))  #62.08336 #321tips
max(branching.times(gobTree))  #55.65662 #54tips
max(branching.times(damTree))  #55.32809 #176tips
max(branching.times(angTree))  #54.76698 #72tips
max(branching.times(sfpTree))  #14.89154 #22tips

#it will be easiest to graft trees in ascending order of family tree size.
#this is because as each tree is grafted, the resulting tree's nodes are 
#re-numbered. accordingly, all 5 trees can't be grafted simultaneouly

#what we need is to first get all the branching times of the nodes
backboneBranchTimes<-branching.times(backbone)
names(backboneBranchTimes)<-seq(from=Ntip(backbone)+1,
                                to=length(backbone$edge.length)+1)
#now look at the relevant nodes and find the first one that is shorter than 
#the surfperch tree height (smallest family tree)
#sfps: 210 has height 13.402837; entry [38]
sfpPosition<-as.numeric(max(branching.times(sfpTree))-backboneBranchTimes[38])
bind1<-bind.tree(backbone,sfpTree,where=210,position=sfpPosition)
#verify
plot(bind1);nodelabels(frame='none')
axisPhylo(side=1,root.time=NULL,backward=TRUE)


#gobies next
bind1BranchTimes<-branching.times(bind1)
names(bind1BranchTimes)<-seq(from=Ntip(bind1)+1,
                                to=length(bind1$edge.length)+1)
#round(bind1BranchTimes,3)
#gobs: 335 has height 49.299; entry [141]
gobPosition<-as.numeric(max(branching.times(gobTree))-bind1BranchTimes[141])
bind2<-bind.tree(bind1,gobTree,where=335,position=gobPosition)
plot(bind2);nodelabels(frame='none')
axisPhylo(side=1,root.time=NULL,backward=TRUE)


#angels next
bind2BranchTimes<-branching.times(bind2)
names(bind2BranchTimes)<-seq(from=Ntip(bind2)+1,
                             to=length(bind2$edge.length)+1)
#round(bind2BranchTimes,3)
#angs: 313 has height 41.717; entry [65]
angPosition<-as.numeric(max(branching.times(angTree))-bind2BranchTimes[65])
bind3<-bind.tree(bind2,angTree,where=313,position=angPosition)
plot(bind3);nodelabels(frame='none')
axisPhylo(side=1,root.time=NULL,backward=TRUE)


#damsels next
bind3BranchTimes<-branching.times(bind3)
names(bind3BranchTimes)<-seq(from=Ntip(bind3)+1,
                             to=length(bind3$edge.length)+1)
#round(bind3BranchTimes,3)
#dams: 470 has height 35.600; entry [150]
damPosition<-as.numeric(max(branching.times(damTree))-bind3BranchTimes[150])
bind4<-bind.tree(bind3,damTree,where=470,position=damPosition)
plot(bind4);nodelabels(frame='none')
axisPhylo(side=1,root.time=NULL,backward=TRUE)


#finally labrids
bind4BranchTimes<-branching.times(bind4)
names(bind4BranchTimes)<-seq(from=Ntip(bind4)+1,
                             to=length(bind4$edge.length)+1)
#round(bind4BranchTimes,3)
#dams: 557 has height 57.366; entry [61]
labPosition<-as.numeric(max(branching.times(labTree))-bind4BranchTimes[61])
bind5<-bind.tree(bind4,labTree,where=556,position=labPosition)
plot(bind5);nodelabels(frame='none')
axisPhylo(side=1,root.time=NULL,backward=TRUE)


##### ultrametry #####
#we'll need to account for small rounding errors to ensure tree is ultrametric
fiveFamTree_unpruned<-force.ultrametric(bind5,method=c("nnls","extend"))
plot(fiveFamTree_unpruned);axisPhylo(side=1,root.time=NULL,backward=TRUE)
is.ultrametric(fiveFamTree_unpruned)


##### pruning #####
#prune out all the original taxa for morphometric-specific tree
fiveFamSorted<-treedata(fiveFamTree_unpruned,all.two.d,sort=TRUE,warnings=FALSE)
ladderize(fiveFamSorted$phy)->fiveFamTree
plot(fiveFamTree);axisPhylo(side=1,root.time=NULL,backward=TRUE)
#this tree is saved in "./tree_prep" as "fiveFamTree.tre"

#prune out all original taxa for an unpruned tree that has all taxa for families
#including those for which we don't have morphometric data. This will be useful
#for stochastic character mapping
#gather all tip labels:
fiveFamALLTIPS<-c(labTree$tip.label,gobTree$tip.label,angTree$tip.label,
                  damTree$tip.label,sfpTree$tip.label)
#generate some fake data to help with the treedata() function
crap<-rnorm(length(fiveFamALLTIPS))
crapdata<-data.frame(phylo=fiveFamALLTIPS,crap)
rownames(crapdata)<-fiveFamALLTIPS
fiveFamALLSorted<-treedata(fiveFamTree_unpruned,crapdata,sort=TRUE,
                           warnings=FALSE)
ladderize(fiveFamALLSorted$phy)->fiveFamTreeALL
plot(fiveFamTreeALL);axisPhylo(side=1,root.time=NULL,backward=TRUE)
max(branching.times(fiveFamTreeALL))
#this tree is saved in "./tree_prep" as "fiveFamTreeALL.tre"